// ExcelReader.java
// author: Michele Sonnessa
// email: sonnessa@di.unito.it

import xlrd.*;
import java.io.*;

/**
  * This class is a simple and a fast way to use the Andy Khan's excelread library.
  * Using ExcelReader you can read data from excel sequentially or specifing positions of
  * the cell to read. <BR>
  * Sequential reading allows you to use excel sheets like sequential text files.
  * Each value is read using the getValue() method without passing to it any parameters.
  * The ExcelReader class gets values one by one until the eof() method returns true. <BR>
  * When you know the structure of your data or you are using a table as input,
  * the direct reading method can be used. You get values asking ExcelRead to return
  * the value of each cell giving its (row, col) position. <BR>
  * The row and column index start both from 0 value.
  *
  * @author Michele Sonnessa
  * @version 0.2.1
  */
public class ExcelReader {

	private Workbook 	currentWorkbook;
	private Sheet 		currentSheet;
	private Cell[] 		row = null;
	private int 		currentX, currentY;
	private int 		maxY, maxX;
	private boolean 	emptySkip;

	// ************
	// CONSTRUCTORS
	// ************

	/**
	 * Opens the file and sets the first excel sheet the current sheet.
	 * During sequential reading empty cells will be skipped.
	 */
    public ExcelReader(String fileName)
	{
		openWorkbook(fileName);
		emptySkip = true;
		currentSheet = currentWorkbook.getSheet(0);
		updateSheetValues();
	}

	/**
	 * Opens the file and sets the first excel sheet the current sheet.<BR>
	 * If skipEmptyCells parameter is set to false, during sequential reading
	 * each empty cell read will be considered as blank string or a 0 value. <BR>
	 * For instance: consider a sequence like this: |3|0| |7| <BR>
	 * If skipEmptyCells is true it returns 3, 0, 7 <BR>
	 * otherwise it returns 3, 0, 0, 7
	 */
    public ExcelReader(String fileName, boolean skipEmptyCells)
	{
		openWorkbook(fileName);
		emptySkip = skipEmptyCells;
		currentSheet = currentWorkbook.getSheet(0);
		updateSheetValues();
	}

	/**
	 * This constuctor allows you to open and to select directly the source sheet.
	 * If the sheet does not exist the program will be terminated printing an error message.<BR>
	 * If skipEmptyCells parameter is set to false, during sequential reading
	 * each empty cell read will be considered as blank string or a 0 value. <BR>
	 * For instance: consider a sequence like this: |3|0| |7| <BR>
	 * If skipEmptyCells is true it returns 3, 0, 7 <BR>
	 * otherwise it returns 3, 0, 0, 7
	 */
        public ExcelReader(String fileName, boolean skipEmptyCells, String sheetName)
	{
		openWorkbook(fileName);
		emptySkip = skipEmptyCells;
		currentSheet = currentWorkbook.getSheet(sheetName);

		if (currentSheet == null)
		{
			System.err.println("ExcelReader Error: in '" + fileName + "' the worksheet '" +
								sheetName + "' was not found!");
			System.exit(0);
		}
		updateSheetValues();
	}

	//Method used by constructors
	private void openWorkbook(String fileName)
	{
		try
		{
			currentWorkbook = Workbook.getWorkbook(new File(fileName));
		}
		catch (ExcelReadException e)
		{
			System.err.println("Trying to read '" + fileName + "': " + e.toString());
		}
		catch (Throwable t)
		{
		  System.out.println(t.toString());
		  t.printStackTrace();
		}
	}


	// **************
	// PUBLIC METHODS
	// **************

	/** Tests if the 'sheetName' sheet is present in the current Excel document.
	 * It is to be used before the selectSheet() method in order to avoid
	 * program termination.
	 */
	public boolean existsSheetWithName(String sheetName)
	{
		return (currentWorkbook.getSheet(sheetName) != null);
	}

	/** Changes the current sheet to the 'sheetName'. If the sheet is not present
	 * the program is terminated. Before using this method is better to test
	 * its presence using the existsSheetWithName() method.
	 */
	public void selectSheet(String sheetName)
	{
		currentSheet = currentWorkbook.getSheet(sheetName);
		if (currentSheet == null)
		{
			System.err.println("ExcelReader Error: the worksheet '" +
								sheetName + "' was not found!");
			System.exit(0);
		}
		updateSheetValues();
	}

	/** Returns the current sheet reference.
	 */
	public Sheet getCurrentSheet()
	{
		return currentSheet;
	}

	//Reading at x,y
	/** Gets the value at (atRow, atCol) position and returns it as a String value. */
	public String getStrValue(int atRow, int atCol)
	{
		changeRow(atRow);
		return readStrValue(atCol);
	}
	/** Gets the value at (atRow, atCol) position and returns it as an int value.
	 * It uses the Interger.parseInt() method to cast the String original value.*/
	public int getIntValue(int atRow, int atCol)
	{
		changeRow(atRow);
		return readIntValue(atCol);
	}
	/** Gets the value at (atRow, atCol) position and returns it as a double value.
	 * It uses the Double.parseDouble() method to cast the String original value.
	 * NOTICE: All ',' chars are automatically converted in '.' chars. [Italian settings]*/
	public double getDblValue(int atRow, int atCol)
	{
		changeRow(atRow);
		return readDblValue(atCol);
	}
	/** Checks if the cell at (atRow, atCol) position contains
	 *  a label (leaving unchanged the counter) */
	public boolean checkForLabelCell(int atRow, int atCol)
	{
		changeRow(atRow);
		return checkLabelOfCell(atCol);
	}

	//Streaming

	/** Gets the next value in reading sequence and returns it as a String value. */
	public String getStrValue()
	{
		return readStrValue(currentX);
	}
	/** Gets the next value in reading sequence and returns it as an int value.
	 * It uses the Interger.parseInt() method to cast the String original value.*/
	public int getIntValue()
	{
		return readIntValue(currentX);
	}
	/** Gets the next value in reading sequence and returns it as a double value.
	 * It uses the Double.parseDouble() method to cast the String original value.*/
	public double getDblValue()
	{
		return readDblValue(currentX);
	}
	/** Checks if the next cell in reading sequence contains
	 *  a label (leaving unchanged the counter) */
	public boolean checkForLabelCell()
	{
			return checkLabelOfCell(currentX);
	}



	/** Tests if reading head reached the end of file (the last element in sequence). */
	public boolean eof()
	{
		return (currentX >= maxX && currentY >= maxY);
	}

	/** Tests if reading head reached the end of line. */
	public boolean eol()
	{
		return (currentX >= maxX);
	}


	// *****************
	// PRIVATE FUNCTIONS
	// *****************
	private void incCounter()
	{
		boolean ok = false;

		while (!ok && !eof())
		{
			currentX++;
			if (currentX > maxX)
			{
				currentX = 0;
				currentY++;
				changeRow(currentY);
			}

			if (!emptySkip)
				ok	= true;
			else
				if (row[currentX].getType() != CellType.EMPTY)
					ok	= true;
		}
	}

    private boolean checkLabelOfCell(int x)
	{
		currentX = x;
		if (eof())
		{
			System.err.println("ExcelReader Error: you were trying to read out of readable area!");
			System.err.println("Sheet '" + currentSheet.getName() + "' at row=" + currentY + " : col=" + currentX);
			System.exit(0);
		}
                if (row[x].getType() == CellType.LABEL)
                        return true;
                else    return false;
	}
	private String readStrValue(int x)
	{
		currentX = x;
		if (eof())
		{
			System.err.println("ExcelReader Error: you were trying to read out of readable area!");
			System.err.println("Sheet '" + currentSheet.getName() + "' at row=" + currentY + " : col=" + currentX);
			System.exit(0);
		}
		String s = row[x].getContents();
		incCounter();
		return s;
	}
	private int readIntValue(int x)
	{
		currentX = x;
		if (eof())
		{
			System.err.println("ExcelReader Error: you were trying to read out of readable area!");
			System.err.println("Sheet '" + currentSheet.getName() + "' at row=" + currentY + " : col=" + currentX);
			System.exit(0);
		}

		try
		{
			int i = Integer.parseInt(row[x].getContents());
			incCounter();
			return i;
		}
		catch (NumberFormatException e)
		{
			incCounter();
			return 0;
		}
	}

	private double readDblValue(int x)
	{
		currentX = x;
		if (eof())
		{
			System.err.println("ExcelReader Error: you were trying to read out of readable area!");
			System.err.println("Sheet '" + currentSheet.getName() + "' at row=" + currentY + " : col=" + currentX);
			System.exit(0);
		}

		try
		{
			double d = Double.parseDouble(row[x].getContents().replace(',', '.'));
			incCounter();
			return d;
		}
		catch (NumberFormatException e)
		{
			incCounter();
			return 0.0;
		}
	}

	private boolean updateSheetValues()
	{
		if (currentSheet == null)
			return false;
		maxY = currentSheet.getRows();
		return changeRow(0);
	}

	private boolean changeRow(int newRow)
	{
		if (currentSheet == null)
			return false;

		currentX = 0;
		currentY = newRow;

		if (newRow >= maxY)
		{
			maxX = 0;
			return false;
		}

		row = currentSheet.getRow(newRow);


		// Find the last non-blank entry in the row
		int nonblank = 0;
		for (int j = row.length - 1; j >= 0 ; j--)
		{
		  if (row[j].getType() != CellType.EMPTY)
		  {
			nonblank = j;
			break;
		  }
		}
		maxX = nonblank;

		return true;
	}

}
